Class: RubyLLM::MCP::Auth::BrowserOAuthProvider::NullLogger

Inherits:
Object
  • Object
show all
Defined in:
lib/ruby_llm/mcp/auth/browser_oauth_provider.rb

Overview

Callback worker thread logs are intentionally isolated from the caller logger. JRuby + rspec-mocks logger doubles are not safe to share across threads.

Instance Method Summary collapse

Instance Method Details

#debugObject



48
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 48

def debug(*) = nil

#errorObject



51
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 51

def error(*) = nil

#infoObject



49
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 49

def info(*) = nil

#warnObject



50
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 50

def warn(*) = nil