Class: RubyLLM::MCP::Auth::BrowserOAuthProvider::NullLogger
- Inherits:
-
Object
- Object
- RubyLLM::MCP::Auth::BrowserOAuthProvider::NullLogger
- Defined in:
- lib/ruby_llm/mcp/auth/browser_oauth_provider.rb
Overview
Callback worker thread logs are intentionally isolated from the caller logger. JRuby + rspec-mocks logger doubles are not safe to share across threads.
Instance Method Summary collapse
Instance Method Details
#debug ⇒ Object
48 |
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 48 def debug(*) = nil |
#error ⇒ Object
51 |
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 51 def error(*) = nil |
#info ⇒ Object
49 |
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 49 def info(*) = nil |
#warn ⇒ Object
50 |
# File 'lib/ruby_llm/mcp/auth/browser_oauth_provider.rb', line 50 def warn(*) = nil |