Class: RubyLLM::MCP::Auth::GrantStrategies::Base
- Inherits:
-
Object
- Object
- RubyLLM::MCP::Auth::GrantStrategies::Base
- Defined in:
- lib/ruby_llm/mcp/auth/grant_strategies/base.rb
Overview
Base strategy for OAuth grant types Defines interface for grant-specific configuration
Direct Known Subclasses
Instance Method Summary collapse
-
#auth_method ⇒ String
Get token endpoint authentication method.
-
#grant_types_list ⇒ Array<String>
Get list of grant types to request during registration.
-
#response_types_list ⇒ Array<String>
Get list of response types to request during registration.
Instance Method Details
#auth_method ⇒ String
Get token endpoint authentication method
12 13 14 |
# File 'lib/ruby_llm/mcp/auth/grant_strategies/base.rb', line 12 def auth_method raise NotImplementedError, "#{self.class} must implement #auth_method" end |
#grant_types_list ⇒ Array<String>
Get list of grant types to request during registration
18 19 20 |
# File 'lib/ruby_llm/mcp/auth/grant_strategies/base.rb', line 18 def grant_types_list raise NotImplementedError, "#{self.class} must implement #grant_types_list" end |
#response_types_list ⇒ Array<String>
Get list of response types to request during registration
24 25 26 |
# File 'lib/ruby_llm/mcp/auth/grant_strategies/base.rb', line 24 def response_types_list raise NotImplementedError, "#{self.class} must implement #response_types_list" end |